#!/usr/bin/env python

__author__ = "Liang Huang"

'''translate the AST parsed by compiler.parse() and generate C code.
   based on Siek's code, but added assignments and for statement.'''

import sys
logs = sys.stderr
import compiler
from compiler.ast import *

def generate_c(n):
    if isinstance(n, Module):
        return "\n".join(["#include <stdio.h>",
                          "int main()",
                          "{", 
                          generate_c(n.node), 
                          "return 0;",
                          "}"])
    elif isinstance(n, Stmt):  # only one statement; TODO: handle multiple statements
        return generate_c(n.nodes[0])
    elif isinstance(n, Printnl): # only support single item; TODO: handle multiple items
        return 'printf(\"%%d\\n\", %s);' % generate_c(n.nodes[0])
    elif isinstance(n, Const):
        return '%d' % n.value
    elif isinstance(n, UnarySub):
        return '(-%s)' % generate_c(n.expr)
    else:
        raise sys.exit('Error in generate_c: unrecognized AST node: %s' % n)

if __name__ == "__main__":        
    try:
        ast = compiler.parse("\n".join(sys.stdin.readlines()))
        print >> logs, ast # debug
        print generate_c(ast)
    except Exception, e:
        print >> logs, e.args[0]
        exit(-1)
